@@ -12,7 +12,8 @@ from pyspark.sql.dataframe import DataFrame
 from pyspark.sql.types import StructType
 
 from data_product_sdk.cortex_databricks.data_platform.aws_utils import S3
-from data_product_sdk.cortex_databricks.data_platform.databricks_utils import reset_autoloader_checkpoint
+from data_product_sdk.cortex_databricks.data_platform.databricks_utils import reset_autoloader_checkpoint, __delete_checkpoint_location
+from data_product_sdk.cortex_databricks.data_platform.kafka_utils import create_topics_if_not_exists, DEFAULT_BOOTSTRAP_SERVERS
 from data_product_sdk.cortex_databricks.data_platform.processing_utils import process_dataframe
 from data_product_sdk.cortex_databricks.data_platform.profilling_utils import calculate_profilling
 from data_product_sdk.cortex_databricks.data_platform.schema_utils import get_schema_from_s3
@@ -89,7 +90,14 @@ def build_table_path(bucket_name: str, data_pack_name: str, dataset_name: str):
     return table_location
 
 
-def build_checkpoint_path(bucket_name: str, data_pack_name: str, dataset_name: str, input_table_name: str = ''):
+def build_checkpoint_path(bucket_name: str, data_pack_name: str, dataset_name: str, input_table_name: str = '', checkpoint_path=None):
+    """
+    :param bucket_name: bucket in S3 where the checkpoint is stored
+    :param data_pack_name: name of the data_pack
+    :param dataset_name: name of the dataset
+    :param input_table_name: input_table if exists
+    :return:
+    """
     assert bucket_name == BRONZE_BUCKET or (input_table_name and bucket_name in (SILVER_BUCKET, GOLD_BUCKET)), \
         f'bucket name must be one of {BRONZE_BUCKET}, {SILVER_BUCKET} or {GOLD_BUCKET} and {input_table_name} must be valid when using {SILVER_BUCKET} or {GOLD_BUCKET}'
     checkpoint_location = f's3://{bucket_name}/_databricks_autoloader_checkpoints/{data_pack_name}/{dataset_name}/{input_table_name}'
@@ -355,6 +363,28 @@ def silver_to_gold(input_table_name, data_pack_name, dataset_name, input_db_name
         apply_profiling(df=df, data_pack_name=data_pack_name, dataset_name=dataset_name)
 
 
+def gold_to_kafka(input_table_name, data_pack_name, dataset_name, topic_name, input_db_name=GOLD_DB_NAME, spark=None, write_options: dict = None,
+                  kafka_bootstrap_servers=DEFAULT_BOOTSTRAP_SERVERS, processing_function=None, processing_function_args=None, processing_function_kwargs=None, reset_checkpoint=False):
+    topics = [topic_name]
+    create_topics_if_not_exists(topics=topics, bootstrap_servers=kafka_bootstrap_servers)
+    checkpoint_location = f's3://{GOLD_BUCKET}/_databricks_autoloader_checkpoints/kafka/{data_pack_name}/{dataset_name}/{input_table_name}'
+    if spark is None:
+        spark = get_spark_session(app_name=dataset_name)
+    if reset_checkpoint:
+        __delete_checkpoint_location(checkpoint_location=checkpoint_location)
+
+    df = read_table(spark, db_name=input_db_name, table_name=input_table_name)
+    df = apply_processing_function(df=df, func=processing_function, func_args=processing_function_args, func_kwargs=processing_function_kwargs)
+
+    stream = df.writeStream.trigger(once=True) \
+        .format('kafka') \
+        .option('checkpointLocation', checkpoint_location) \
+        .option("kafka.bootstrap.servers", kafka_bootstrap_servers) \
+        .option("topic", topic_name) \
+        .options(**write_options)
+    stream.start()
+
+
 if __name__ == '__main__':
     data_input_path = 's3://cortex-data-platform-trusted-area-dev/salesdata/year=2022/month=04/day=22/c643c9d88fec4e39b5a7539c7ed966d7.csv'
     reset_autoloader_checkpoint(data_input_path)
